@@ -2,6 +2,7 @@
 
 #include <sys/stat.h>
 
+#include <inttypes.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,7 +40,7 @@ print_id(const char *keyfilename)
 	uint64_t machinenum = (uint64_t)(-1);
 
 	/* Read keyfile and machine name. */
-	if (keyfile_read(keyfilename, &machinenum, ~0)) {
+	if (keyfile_read(keyfilename, &machinenum, ~0, 0)) {
 		warnp("Cannot read key file: %s", keyfilename);
 		exit(1);
 	}
@@ -50,7 +51,7 @@ print_id(const char *keyfilename)
 
 	/* NOTREACHED */
 }
-	
+
 static void
 print_permissions(const char *keyfilename)
 {
@@ -60,7 +61,7 @@ print_permissions(const char *keyfilename)
 	int has_delete;
 
 	/* Read keyfile and machine name. */
-	if (keyfile_read(keyfilename, &machinenum, ~0)) {
+	if (keyfile_read(keyfilename, &machinenum, ~0, 0)) {
 		warnp("Cannot read key file: %s", keyfilename);
 		exit(1);
 	}
@@ -182,6 +183,11 @@ main(int argc, char **argv)
 				    " argument: %s", optarg);
 				exit(1);
 			}
+			if (maxmem > SIZE_MAX) {
+				fprintf(stderr,
+				    "Passphrase memory size is too large\n");
+				exit(1);
+			}
 			break;
 		GETOPT_OPTARG("--passphrase-time"):
 			if (maxtime != 1.0)
@@ -268,7 +274,7 @@ main(int argc, char **argv)
 		 * keys we want to copy, but there's no point really since we
 		 * export keys selectively.
 		 */
-		if (keyfile_read(argv[0], &kfmachinenum, ~0)) {
+		if (keyfile_read(argv[0], &kfmachinenum, ~0, 0)) {
 			warnp("Cannot read key file: %s", argv[0]);
 			exit(1);
 		}
@@ -310,7 +316,7 @@ main(int argc, char **argv)
 
 	/* Write out new key file. */
 	if (keyfile_write(newkeyfile, machinenum, keyswanted,
-	    passphrase, maxmem, maxtime))
+	    passphrase, (size_t)maxmem, maxtime))
 		exit(1);
 
 	/* Success! */
