@@ -137,7 +137,7 @@ static int
 compare(const struct pnode * n, const uint8_t * key, size_t keylen,
     uint8_t * mlen, uint8_t * mask)
 {
-	size_t	i;
+	uint8_t i;
 	uint8_t	mm;
 
 	/* Scan through the complete bytes in the node. */
@@ -322,7 +322,7 @@ patricia_insert(PATRICIA * P, const uint8_t * key, size_t keylen, void * rec)
 				slen = MAXSLEN;
 
 			/* Create the node or error out. */
-			if ((pnew = node_alloc(slen, key)) == NULL)
+			if ((pnew = node_alloc((uint8_t)slen, key)) == NULL)
 				return (-1);
 
 			/* Add the new node into the tree. */
@@ -340,7 +340,7 @@ patricia_insert(PATRICIA * P, const uint8_t * key, size_t keylen, void * rec)
 			 */
 			/* Create the lower of the new nodes. */
 			slen = (*np)->slen - mlen;
-			pnew2 = node_dup(*np, slen, (*np)->s + mlen);
+			pnew2 = node_dup(*np, (uint8_t)slen, (*np)->s + mlen);
 			if (pnew2 == NULL)
 				return (-1);
 
