@@ -17,6 +17,12 @@ typedef struct rwhashtab_internal RWHASHTAB;
  */
 RWHASHTAB * rwhashtab_init(size_t, size_t);
 
+/**
+ * rwhashtab_getsize(table):
+ * Return the number of entries in the table.
+ */
+size_t rwhashtab_getsize(RWHASHTAB *);
+
 /**
  * rwhashtab_insert(table, record):
  * Insert the provided record into the hash table.  Returns (-1) on error,
