@@ -116,7 +116,7 @@ netproto_keyexchange(NETPROTO_CONNECTION * C, const char * useragent,
 		goto err0;
 	if ((KC->useragent = strdup(useragent)) == NULL)
 		goto err1;
-	KC->useragentlen = useragentlen;
+	KC->useragentlen = (uint8_t)useragentlen;
 	KC->callback = callback;
 	KC->cookie = cookie;
 	KC->C = C;
@@ -371,7 +371,7 @@ dh_sent(void * cookie, int status)
 	 * better as long as latency costs less than $45/hour.
 	 *
 	 * Since I don't expect people to be running tarsnap from mobile
-	 * phones (or over 19.2Kbps modems, where the added bandwidth slows
+	 * phones (or over 19.2kbps modems, where the added bandwidth slows
 	 * down the connection by longer than the overlapping of computation
 	 * time saves) any time soon, I'm taking the route which optimizes
 	 * for time.
