@@ -26,6 +26,11 @@
  * This file was originally written by Colin Percival as part of the Tarsnap
  * online backup system.
  */
+
+/* We use non-POSIX functionality in this file. */
+#undef _POSIX_C_SOURCE
+#undef _XOPEN_SOURCE
+
 #include "bsdtar_platform.h"
 
 #include <sys/types.h>
@@ -92,7 +97,7 @@ memlimit_sysctl_hw(size_t * memlimit, int mibleaf)
 	if (sysctlval > SIZE_MAX)
 		*memlimit = SIZE_MAX;
 	else
-		*memlimit = sysctlval;
+		*memlimit = (size_t)sysctlval;
 #else
 	*memlimit = sysctlval;
 #endif
@@ -134,7 +139,7 @@ memlimit_sysinfo(size_t * memlimit)
 	if (totalmem > SIZE_MAX)
 		*memlimit = SIZE_MAX;
 	else
-		*memlimit = totalmem;
+		*memlimit = (size_t)totalmem;
 #else
 	*memlimit = totalmem;
 #endif
@@ -159,7 +164,7 @@ memlimit_rlimit(size_t * memlimit)
 		return (1);
 	if ((rl.rlim_cur != RLIM_INFINITY) &&
 	    ((uint64_t)rl.rlim_cur < memrlimit))
-		memrlimit = rl.rlim_cur;
+		memrlimit = (uint64_t)rl.rlim_cur;
 #endif
 
 	/* ... RLIMIT_DATA... */
@@ -167,7 +172,7 @@ memlimit_rlimit(size_t * memlimit)
 		return (1);
 	if ((rl.rlim_cur != RLIM_INFINITY) &&
 	    ((uint64_t)rl.rlim_cur < memrlimit))
-		memrlimit = rl.rlim_cur;
+		memrlimit = (uint64_t)rl.rlim_cur;
 
 	/* ... and RLIMIT_RSS. */
 #ifdef RLIMIT_RSS
@@ -175,7 +180,7 @@ memlimit_rlimit(size_t * memlimit)
 		return (1);
 	if ((rl.rlim_cur != RLIM_INFINITY) &&
 	    ((uint64_t)rl.rlim_cur < memrlimit))
-		memrlimit = rl.rlim_cur;
+		memrlimit = (uint64_t)rl.rlim_cur;
 #endif
 
 	/* Return the value, but clamp to SIZE_MAX if necessary. */
@@ -183,7 +188,7 @@ memlimit_rlimit(size_t * memlimit)
 	if (memrlimit > SIZE_MAX)
 		*memlimit = SIZE_MAX;
 	else
-		*memlimit = memrlimit;
+		*memlimit = (size_t)memrlimit;
 #else
 	*memlimit = memrlimit;
 #endif
@@ -212,8 +217,11 @@ memlimit_sysconf(size_t * memlimit)
 	/* Read the two limits. */
 	if (((pagesize = sysconf(_SC_PAGE_SIZE)) == -1) ||
 	    ((physpages = sysconf(_SC_PHYS_PAGES)) == -1)) {
-		/* Did an error occur? */
-		if (errno != 0)
+		/*
+		 * Did an error occur?  OS X may return EINVAL due to not
+		 * supporting _SC_PHYS_PAGES in spite of defining it.
+		 */
+		if (errno != 0 && errno != EINVAL)
 			return (1);
 
 		/* If not, there is no limit. */
@@ -228,7 +236,7 @@ memlimit_sysconf(size_t * memlimit)
 	if (totalmem > SIZE_MAX)
 		*memlimit = SIZE_MAX;
 	else
-		*memlimit = totalmem;
+		*memlimit = (size_t)totalmem;
 #else
 	*memlimit = totalmem;
 #endif
@@ -298,7 +306,7 @@ memtouse(size_t maxmem, double maxmemfrac, size_t * memlimit)
 	/* Only use the specified fraction of the available memory. */
 	if ((maxmemfrac > 0.5) || (maxmemfrac == 0.0))
 		maxmemfrac = 0.5;
-	memavail = maxmemfrac * memlimit_min;
+	memavail = (size_t)(maxmemfrac * memlimit_min);
 
 	/* Don't use more than the specified maximum. */
 	if ((maxmem > 0) && (memavail > maxmem))
