@@ -2,23 +2,22 @@
 
 #include <sys/time.h>
 
+#include "monoclock.h"
 #include "warnp.h"
 
 #include "tvmath.h"
 
 /**
  * tvmath_addctime(tv):
- * Set tv += gettimeofday.
+ * Set tv += monoclock.
  */
 int
 tvmath_addctime(struct timeval * tv)
 {
 	struct timeval tnow;
 
-	if (gettimeofday(&tnow, NULL)) {
-		warnp("gettimeofday");
+	if (monoclock_get(&tnow))
 		goto err0;
-	}
 	tv->tv_sec += tnow.tv_sec;
 	tv->tv_usec += tnow.tv_usec;
 	if (tv->tv_usec >= 1000000) {
@@ -36,17 +35,15 @@ err0:
 
 /**
  * tvmath_subctime(tv):
- * Set tv -= gettimeofday.
+ * Set tv -= monoclock.
  */
 int
 tvmath_subctime(struct timeval * tv)
 {
 	struct timeval tnow;
 
-	if (gettimeofday(&tnow, NULL)) {
-		warnp("gettimeofday");
+	if (monoclock_get(&tnow))
 		goto err0;
-	}
 	tv->tv_sec -= tnow.tv_sec;
 	tv->tv_usec -= tnow.tv_usec;
 	if (tv->tv_usec < 0) {
@@ -64,17 +61,15 @@ err0:
 
 /**
  * tvmath_rsubctime(tv):
- * Set tv = gettimeofday - tv.
+ * Set tv = monoclock - tv.
  */
 int
 tvmath_rsubctime(struct timeval * tv)
 {
 	struct timeval tnow;
 
-	if (gettimeofday(&tnow, NULL)) {
-		warnp("gettimeofday");
+	if (monoclock_get(&tnow))
 		goto err0;
-	}
 	tv->tv_sec = tnow.tv_sec - tv->tv_sec;
 	tv->tv_usec = tnow.tv_usec - tv->tv_usec;
 	if (tv->tv_usec < 0) {
