@@ -79,6 +79,7 @@ __FBSDID("$FreeBSD: src/usr.bin/tar/bsdtar.c,v 1.93 2008/11/08 04:43:24 kientzle
 
 #include "bsdtar.h"
 #include "crypto.h"
+#include "dirutil.h"
 #include "humansize.h"
 #include "keyfile.h"
 #include "tarsnap_opt.h"
@@ -105,11 +106,12 @@ time_t get_date(time_t, const char *);
 static struct bsdtar	*bsdtar_init(void);
 static void		 bsdtar_atexit(void);
 
-static void		 build_dir(struct bsdtar *, const char *dir,
-			     const char *diropt);
-static void		 configfile(struct bsdtar *, const char *fname);
+static void		 configfile(struct bsdtar *, const char *fname,
+			     int fromcmdline);
 static int		 configfile_helper(struct bsdtar *bsdtar,
 			     const char *line);
+static int		 archive_names_helper(struct bsdtar *bsdtar,
+			     const char *line);
 static void		 dooption(struct bsdtar *, const char *,
 			     const char *, int);
 static int		 load_keys(struct bsdtar *, const char *path);
@@ -168,6 +170,14 @@ bsdtar_init(void)
 	bsdtar->siginfo = NULL;
 	bsdtar->substitution = NULL;
 	bsdtar->keyfile = NULL;
+	bsdtar->conffile = NULL;
+	bsdtar->conf_opt = NULL;
+	bsdtar->conf_arg = NULL;
+	bsdtar->conffile_actual = NULL;
+	bsdtar->conffile_buffer = NULL;
+
+	/* Initialize temporary tapenames array. */
+	bsdtar->tapenames_setup = strlist_init(0);
 
 	/* We don't have bsdtar->progname yet, so we can't use bsdtar_errc. */
 	if (atexit(bsdtar_atexit)) {
@@ -182,9 +192,23 @@ static void
 bsdtar_atexit(void)
 {
 	struct bsdtar *bsdtar;
+	size_t i;
 
 	bsdtar = &bsdtar_storage;
 
+	/* Free temporary archive names (if an error occurred before export). */
+	if (bsdtar->tapenames_setup != NULL) {
+		for (i = 0; i < strlist_getsize(bsdtar->tapenames_setup); i++)
+			free(*strlist_get(bsdtar->tapenames_setup, i));
+		strlist_free(bsdtar->tapenames_setup);
+	}
+
+	/* Free arrays containined strings allocated by strdup. */
+	if (bsdtar->tapenames != NULL) {
+		for (i = 0; i < bsdtar->ntapes; i++)
+			free(bsdtar->tapenames[i]);
+	}
+
 	/* Free arrays allocated by malloc. */
 	free(bsdtar->tapenames);
 	free(bsdtar->configfiles);
@@ -194,6 +218,15 @@ bsdtar_atexit(void)
 	free(bsdtar->homedir);
 	free(bsdtar->option_csv_filename);
 	free(bsdtar->keyfile);
+	free(bsdtar->conffile);
+	free(bsdtar->conf_opt);
+	free(bsdtar->conf_arg);
+
+	/* Free file-parsing variables from util.c. */
+	free(bsdtar->conffile_buffer);
+	if ((bsdtar->conffile_actual != NULL) &&
+	    (bsdtar->conffile_actual != stdin))
+		fclose(bsdtar->conffile_actual);
 
 	/* Free matching and (if applicable) substitution patterns. */
 	cleanup_exclusions(bsdtar);
@@ -214,10 +247,10 @@ main(int argc, char **argv)
 	char			 buff[16];
 	char			 cachedir[PATH_MAX + 1];
 	struct passwd		*pws;
-	char			*conffile;
 	const char		*missingkey;
 	time_t			 now;
 	size_t 			 i;
+	char			*tapename_cmdline;
 
 	WARNP_INIT;
 
@@ -249,11 +282,6 @@ main(int argc, char **argv)
 	/* We don't have a machine # yet. */
 	bsdtar->machinenum = (uint64_t)(-1);
 
-	/* Allocate space for archive names; at most argc of them. */
-	if ((bsdtar->tapenames = malloc(argc * sizeof(const char *))) == NULL)
-		bsdtar_errc(bsdtar, 1, ENOMEM, "Cannot allocate memory");
-	bsdtar->ntapes = 0;
-
 	/* Allocate space for config file names; at most argc of them. */
 	if ((bsdtar->configfiles = malloc(argc * sizeof(const char *))) == NULL)
 		bsdtar_errc(bsdtar, 1, ENOMEM, "Cannot allocate memory");
@@ -344,6 +372,18 @@ main(int argc, char **argv)
 		case OPTION_AGGRESSIVE_NETWORKING: /* tarsnap */
 			optq_push(bsdtar, "aggressive-networking", NULL);
 			break;
+		case OPTION_ARCHIVE_NAMES:
+			if (bsdtar->option_archive_names_set)
+				bsdtar_errc(bsdtar, 1, errno,
+				    "Two --archive-names options given");
+			if (bsdtar->optarg == NULL)
+				bsdtar_errc(bsdtar, 1, 0,
+				    "Argument required for --archive-names");
+			bsdtar->option_archive_names_set = 1;
+
+			/* Read tapenames from --archive_names file. */
+			process_lines(bsdtar, bsdtar->optarg,
+			    archive_names_helper, 0);
 		case 'B': /* GNU tar */
 			/* libarchive doesn't need this; just ignore it. */
 			break;
@@ -385,7 +425,7 @@ main(int argc, char **argv)
 		case OPTION_CSV_FILE: /* tarsnap */
 			if (bsdtar->option_csv_filename != NULL)
 				bsdtar_errc(bsdtar, 1, errno,
-				    "Two --csv-file options given.\n");
+				    "Two --csv-file options given");
 			if ((bsdtar->option_csv_filename = strdup(
 			    bsdtar->optarg)) == NULL)
 				bsdtar_errc(bsdtar, 1, errno, "Out of memory");
@@ -403,7 +443,14 @@ main(int argc, char **argv)
 			optq_push(bsdtar, "exclude", bsdtar->optarg);
 			break;
 		case 'f': /* multitar */
-			bsdtar->tapenames[bsdtar->ntapes++] = bsdtar->optarg;
+			if ((tapename_cmdline = strdup(bsdtar->optarg)) == NULL)
+				bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+			if (strlist_append(bsdtar->tapenames_setup,
+			    &tapename_cmdline, 1))
+				bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+			break;
+		case OPTION_FORCE_RESOURCES:
+			optq_push(bsdtar, "force-resources", NULL);
 			break;
 		case OPTION_FSCK: /* multitar */
 			set_mode(bsdtar, opt, "--fsck");
@@ -442,9 +489,15 @@ main(int argc, char **argv)
 		case OPTION_INCLUDE:
 			optq_push(bsdtar, "include", bsdtar->optarg);
 			break;
+		case OPTION_INITIALIZE_CACHEDIR:
+			set_mode(bsdtar, opt, "--initialize-cachedir");
+			break;
 		case OPTION_INSANE_FILESYSTEMS:
 			optq_push(bsdtar, "insane-filesystems", NULL);
 			break;
+		case OPTION_ISO_DATES:
+			optq_push(bsdtar, "iso-dates", NULL);
+			break;
 		case 'k': /* GNU tar */
 			bsdtar->extract_flags |= ARCHIVE_EXTRACT_NO_OVERWRITE;
 			break;
@@ -547,12 +600,17 @@ main(int argc, char **argv)
 		case OPTION_NO_DISK_PAUSE:
 			optq_push(bsdtar, "no-disk-pause", NULL);
 			break;
+		case OPTION_NO_FORCE_RESOURCES:
+			optq_push(bsdtar, "no-force-resources", NULL);
 		case OPTION_NO_HUMANIZE_NUMBERS:
 			optq_push(bsdtar, "no-humanize-numbers", NULL);
 			break;
 		case OPTION_NO_INSANE_FILESYSTEMS:
 			optq_push(bsdtar, "no-insane-filesystems", NULL);
 			break;
+		case OPTION_NO_ISO_DATES:
+			optq_push(bsdtar, "no-iso-dates", NULL);
+			break;
 		case OPTION_NO_MAXBW:
 			optq_push(bsdtar, "no-maxbw", NULL);
 			break;
@@ -745,8 +803,14 @@ main(int argc, char **argv)
 	 * few bytes off anyway since the command line, including "--dry-run"
 	 * is included in the metadata.
 	 */
-	if (bsdtar->option_dryrun && (bsdtar->ntapes == 0))
-		bsdtar->tapenames[bsdtar->ntapes++] = "(dry-run)";
+	if (bsdtar->option_dryrun &&
+	    (strlist_getsize(bsdtar->tapenames_setup) == 0)) {
+		if ((tapename_cmdline = strdup("(dry-run)")) == NULL)
+			bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+		if (strlist_append(bsdtar->tapenames_setup,
+		    &tapename_cmdline, 1))
+			bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+	}
 
 	/* At this point we must have a mode set. */
 	if (bsdtar->mode == '\0')
@@ -764,26 +828,33 @@ main(int argc, char **argv)
 
 	/* Process config files passed on the command line. */
 	for (i = 0; i < bsdtar->nconfigfiles; i++)
-		configfile(bsdtar, bsdtar->configfiles[i]);
+		configfile(bsdtar, bsdtar->configfiles[i], 1);
 
 	/* If we do not have --no-default-config, process default configs. */
 	if (bsdtar->option_no_default_config == 0) {
 		/* Process options from ~/.tarsnaprc. */
 		if (bsdtar->homedir != NULL) {
-			if (asprintf(&conffile, "%s/.tarsnaprc",
+			if (asprintf(&bsdtar->conffile, "%s/.tarsnaprc",
 			    bsdtar->homedir) == -1)
 				bsdtar_errc(bsdtar, 1, errno, "No memory");
 
-			configfile(bsdtar, conffile);
+			configfile(bsdtar, bsdtar->conffile, 0);
 
 			/* Free string allocated by asprintf. */
-			free(conffile);
+			free(bsdtar->conffile);
+			bsdtar->conffile = NULL;
 		}
 
 		/* Process options from system-wide tarsnap.conf. */
-		configfile(bsdtar, ETC_TARSNAP_CONF);
+		configfile(bsdtar, ETC_TARSNAP_CONF, 0);
 	}
 
+	/* Extract tapenames from tapenames_setup. */
+	if (strlist_export(bsdtar->tapenames_setup, &bsdtar->tapenames,
+	    &bsdtar->ntapes))
+		bsdtar_errc(bsdtar, 1, 0, "Out of memory");
+	bsdtar->tapenames_setup = NULL;
+
 	/* Continue with more sanity-checking. */
 	if ((bsdtar->ntapes == 0) &&
 	    (bsdtar->mode != OPTION_PRINT_STATS &&
@@ -791,6 +862,7 @@ main(int argc, char **argv)
 	     bsdtar->mode != OPTION_RECOVER &&
 	     bsdtar->mode != OPTION_FSCK &&
 	     bsdtar->mode != OPTION_FSCK_PRUNE &&
+	     bsdtar->mode != OPTION_INITIALIZE_CACHEDIR &&
 	     bsdtar->mode != OPTION_NUKE &&
 	     bsdtar->mode != OPTION_VERIFY_CONFIG))
 		bsdtar_errc(bsdtar, 1, 0,
@@ -815,6 +887,7 @@ main(int argc, char **argv)
 	     bsdtar->mode == OPTION_RECOVER ||
 	     bsdtar->mode == OPTION_FSCK ||
 	     bsdtar->mode == OPTION_FSCK_PRUNE ||
+	     bsdtar->mode == OPTION_INITIALIZE_CACHEDIR ||
 	     bsdtar->mode == OPTION_PRINT_STATS))
 		bsdtar_errc(bsdtar, 1, 0,
 		    "Cache directory must be specified for %s",
@@ -919,7 +992,9 @@ main(int argc, char **argv)
 	 * necessary since the tar code can change directories.
 	 */
 	if (bsdtar->cachedir != NULL) {
-		build_dir(bsdtar, bsdtar->cachedir, "--cachedir");
+		if (build_dir(bsdtar->cachedir, "--cachedir") != 0)
+			bsdtar_errc(bsdtar, 1, 0,
+			    "Failed to ensure that cachedir exists");
 		if (realpath(bsdtar->cachedir, cachedir) == NULL)
 			bsdtar_errc(bsdtar, 1, errno, "realpath(%s)",
 			    bsdtar->cachedir);
@@ -1041,6 +1116,9 @@ main(int argc, char **argv)
 	case OPTION_FSCK_WRITE:
 		tarsnap_mode_fsck(bsdtar, 0, 0);
 		break;
+	case OPTION_INITIALIZE_CACHEDIR:
+		tarsnap_mode_initialize_cachedir(bsdtar);
+		break;
 	case OPTION_PRINT_STATS:
 		tarsnap_mode_print_stats(bsdtar);
 		break;
@@ -1197,62 +1275,9 @@ long_help(struct bsdtar *bsdtar)
 	version();
 }
 
-static void
-build_dir(struct bsdtar *bsdtar, const char *dir, const char *diropt)
-{
-	struct stat sb;
-	char * s;
-	const char * dirseppos;
-
-	/* We need a directory name and the config option. */
-	assert(dir != NULL);
-	assert(diropt != NULL); 
-
-	/* Move through *dir and build all parent directories. */
-	for (dirseppos = dir; *dirseppos != '\0'; ) {
-		/* Move to the next '/', or the end of the string. */
-		if ((dirseppos = strchr(dirseppos + 1, '/')) == NULL)
-			dirseppos = dir + strlen(dir);
-
-		/* Generate a string containing the parent directory. */
-		if (asprintf(&s, "%.*s", (int)(dirseppos - dir), dir) == -1)
-			bsdtar_errc(bsdtar, 1, errno, "No Memory");
-
-		/* Does the parent directory exist already? */
-		if (stat(s, &sb) == 0)
-			goto nextdir;
-
-		/* Did something go wrong? */
-		if (errno != ENOENT)
-			bsdtar_errc(bsdtar, 1, errno, "stat(%s)", s);
-
-		/* Create the directory. */
-		if (mkdir(s, 0700))
-			bsdtar_errc(bsdtar, 1, errno, "error creating %s", s);
-
-		/* Tell the user what we did. */
-		fprintf(stderr, "Directory %s created for \"%s %s\"\n",
-		    s, diropt, dir);
-
-nextdir:
-		free(s);
-	}
-
-	/* Make sure permissions on the directory are correct. */
-	if (stat(dir, &sb))
-		bsdtar_errc(bsdtar, 1, errno, "stat(%s)", dir);
-	if (sb.st_mode & (S_IRWXG | S_IRWXO)) {
-		if (chmod(dir, sb.st_mode & ~(S_IRWXG | S_IRWXO))) {
-			bsdtar_errc(bsdtar, 1, errno,
-			    "Cannot sanitize permissions on directory: %s",
-			    dir);
-		}
-	}
-}
-
 /* Process options from the specified file, if it exists. */
 static void
-configfile(struct bsdtar *bsdtar, const char *fname)
+configfile(struct bsdtar *bsdtar, const char *fname, int fromcmdline)
 {
 	struct stat sb;
 
@@ -1266,10 +1291,20 @@ configfile(struct bsdtar *bsdtar, const char *fname)
 	bsdtar->option_no_config_include_set =
 	    bsdtar->option_no_config_include;
 
-	/* If the file doesn't exist, do nothing. */
 	if (stat(fname, &sb)) {
-		if (errno == ENOENT)
-			return;
+		/* Missing file. */
+		if (errno == ENOENT) {
+			if (fromcmdline) {
+				bsdtar_errc(bsdtar, 1, errno,
+				    "Cannot read config file: %s", fname);
+			} else {
+				/*
+				 * If the file wasn't specified on the
+				 * command-line, do nothing.
+				 */
+				return;
+			}
+		}
 
 		/*
 		 * Something bad happened.  Note that this could occur if
@@ -1292,10 +1327,8 @@ configfile(struct bsdtar *bsdtar, const char *fname)
 static int
 configfile_helper(struct bsdtar *bsdtar, const char *line)
 {
-	char * conf_opt;
 	char * conf_arg;
 	size_t optlen;
-	char * conf_arg_malloced;
 	size_t len;
 
 	/* Skip any leading whitespace. */
@@ -1307,7 +1340,7 @@ configfile_helper(struct bsdtar *bsdtar, const char *line)
 		return (0);
 
 	/* Duplicate line. */
-	if ((conf_opt = strdup(line)) == NULL)
+	if ((bsdtar->conf_opt = strdup(line)) == NULL)
 		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
 
 	/*
@@ -1315,24 +1348,25 @@ configfile_helper(struct bsdtar *bsdtar, const char *line)
 	 * duplication, but to reduce the number of diffs to a later version,
 	 * we'll do it here.
 	 */
-	len = strlen(conf_opt);
+	len = strlen(bsdtar->conf_opt);
 	if ((len > 0) &&
-	    ((conf_opt[len - 1] == ' ') || (conf_opt[len - 1] == '\t'))) {
+	    ((bsdtar->conf_opt[len - 1] == ' ') ||
+	     (bsdtar->conf_opt[len - 1] == '\t'))) {
 		bsdtar_warnc(bsdtar, 0,
 		    "option contains trailing whitespace; future behaviour"
 		    " may change for:\n    %s", line);
 	}
 
 	/* Split line into option and argument if possible. */
-	optlen = strcspn(conf_opt, " \t");
+	optlen = strcspn(bsdtar->conf_opt, " \t");
 
 	/* Is there an argument? */
-	if (conf_opt[optlen]) {
+	if (bsdtar->conf_opt[optlen]) {
 		/* NUL-terminate the option name. */
-		conf_opt[optlen] = '\0';
+		bsdtar->conf_opt[optlen] = '\0';
 
 		/* Find the start of the argument. */
-		conf_arg = conf_opt + optlen + 1;
+		conf_arg = bsdtar->conf_opt + optlen + 1;
 		conf_arg += strspn(conf_arg, " \t");
 
 		/*
@@ -1353,28 +1387,52 @@ configfile_helper(struct bsdtar *bsdtar, const char *line)
 	if ((conf_arg != NULL) && (conf_arg[0] == '~') &&
 	    (bsdtar->homedir != NULL)) {
 		/* Construct expanded argument string. */
-		if (asprintf(&conf_arg_malloced, "%s%s",
+		if (asprintf(&bsdtar->conf_arg, "%s%s",
 		    bsdtar->homedir, &conf_arg[1]) == -1)
 			bsdtar_errc(bsdtar, 1, errno, "Out of memory");
 
 		/* Use the expanded argument string hereafter. */
-		conf_arg = conf_arg_malloced;
+		conf_arg = bsdtar->conf_arg;
 	} else {
-		conf_arg_malloced = NULL;
+		bsdtar->conf_arg = NULL;
 	}
 
 	/* Process the configuration option. */
-	dooption(bsdtar, conf_opt, conf_arg, 1);
+	dooption(bsdtar, bsdtar->conf_opt, conf_arg, 1);
 
 	/* Free expanded argument or NULL. */
-	free(conf_arg_malloced);
+	free(bsdtar->conf_arg);
+	bsdtar->conf_arg = NULL;
 
 	/* Free memory allocated by strdup. */
-	free(conf_opt);
+	free(bsdtar->conf_opt);
+	bsdtar->conf_opt = NULL;
 
 	return (0);
 }
 
+/* Process a line of configuration file. */
+static int
+archive_names_helper(struct bsdtar *bsdtar, const char *line)
+{
+	char * name;
+
+	/* Ignore blank lines. */
+	if (line[0] == '\0')
+		return (0);
+
+	/* Duplicate line. */
+	if ((name = strdup(line)) == NULL)
+		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+
+	/* Record archive name. */
+	if (strlist_append(bsdtar->tapenames_setup, &name, 1))
+		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+
+	/* Success! */
+	return (0);
+}
+
 /* Add a command-line option to the delayed options queue. */
 static void
 optq_push(struct bsdtar *bsdtar, const char * opt_name, const char * opt_arg)
@@ -1476,6 +1534,12 @@ dooption(struct bsdtar *bsdtar, const char * conf_opt,
 		if (exclude(bsdtar, conf_arg))
 			bsdtar_errc(bsdtar, 1, 0,
 			    "Couldn't exclude %s", conf_arg);
+	} else if (strcmp(conf_opt, "force-resources") == 0) {
+		if (bsdtar->option_force_resources_set)
+			goto optset;
+
+		bsdtar->option_force_resources = 1;
+		bsdtar->option_force_resources_set = 1;
 	} else if (strcmp(conf_opt, "humanize-numbers") == 0) {
 		if (bsdtar->option_humanize_numbers_set)
 			goto optset;
@@ -1497,6 +1561,12 @@ dooption(struct bsdtar *bsdtar, const char * conf_opt,
 
 		bsdtar->option_insane_filesystems = 1;
 		bsdtar->option_insane_filesystems_set = 1;
+	} else if (strcmp(conf_opt, "iso-dates") == 0) {
+		if (bsdtar->option_iso_dates_set)
+			goto optset;
+
+		bsdtar->option_iso_dates = 1;
+		bsdtar->option_iso_dates_set = 1;
 	} else if (strcmp(conf_opt, "keyfile") == 0) {
 		if (bsdtar->keyfile != NULL)
 			goto optset;
@@ -1590,6 +1660,11 @@ dooption(struct bsdtar *bsdtar, const char * conf_opt,
 			goto optset;
 
 		bsdtar->option_disk_pause_set = 1;
+	} else if (strcmp(conf_opt, "no-force-resources") == 0) {
+		if (bsdtar->option_force_resources_set)
+			goto optset;
+
+		bsdtar->option_force_resources_set = 1;
 	} else if (strcmp(conf_opt, "no-humanize-numbers") == 0) {
 		if (bsdtar->option_humanize_numbers_set)
 			goto optset;
@@ -1600,6 +1675,11 @@ dooption(struct bsdtar *bsdtar, const char * conf_opt,
 			goto optset;
 
 		bsdtar->option_insane_filesystems_set = 1;
+	} else if (strcmp(conf_opt, "no-iso-dates") == 0) {
+		if (bsdtar->option_iso_dates_set)
+			goto optset;
+
+		bsdtar->option_iso_dates_set = 1;
 	} else if (strcmp(conf_opt, "no-maxbw") == 0) {
 		if (bsdtar->option_maxbw_set)
 			goto optset;
@@ -1746,7 +1826,7 @@ load_keys(struct bsdtar *bsdtar, const char *path)
 	uint64_t machinenum;
 
 	/* Load the key file. */
-	if (keyfile_read(path, &machinenum, ~0))
+	if (keyfile_read(path, &machinenum, ~0, bsdtar->option_force_resources))
 		goto err0;
 
 	/* Check the machine number. */
