@@ -35,6 +35,10 @@
 #include <stdint.h>
 #include <stdio.h>
 
+#include "elasticarray.h"
+
+ELASTICARRAY_DECL(STRLIST, strlist, char *);
+
 /*
  * The internal state for the "tarsnap" program.
  *
@@ -45,7 +49,7 @@
  */
 struct bsdtar {
 	/* Options */
-	const char	**tapenames; /* -f tapename */
+	char		**tapenames; /* -f tapename */
 	size_t		  ntapes;
 	char		 *homedir;
 	char		 *cachedir; /* --cachedir */
@@ -91,6 +95,7 @@ struct bsdtar {
 	double		  bwlimit_rate_down;	/* --maxbw-rate(-down)? */
 	int		  disk_pause;		/* --disk-pause */
 	int		  option_aggressive_networking_set;
+	int		  option_archive_names_set;
 	int		  option_cachecrunch_set;
 	int		  option_disk_pause_set;
 	int		  option_humanize_numbers_set;
@@ -111,6 +116,10 @@ struct bsdtar {
 	int		  option_retry_forever_set;
 	int		  option_insane_filesystems;
 	int		  option_insane_filesystems_set;
+	int		  option_iso_dates;
+	int		  option_iso_dates_set;
+	int		  option_force_resources;
+	int		  option_force_resources_set;
 	const char	**configfiles;		/* --configfile */
 	size_t 		  nconfigfiles;
 	int		  option_no_default_config; /* --no-default-config */
@@ -130,6 +139,16 @@ struct bsdtar {
 	char		  warned_lead_slash; /* Already displayed warning */
 	char		  next_line_is_dir; /* Used for -C parsing in -cT */
 
+	/* Config-file parsing strings. */
+	char		 *conffile;
+	char		 *conf_opt;
+	char		 *conf_arg;
+	FILE		 *conffile_actual;
+	char		 *conffile_buffer;
+
+	/* Temporary array for loading archive names. */
+	STRLIST		 tapenames_setup;
+
 	/* Used for --dryrun with tarsnap.conf.sample with a missing keyfile. */
 	int		  config_file_keyfile_failed;
 
@@ -166,6 +185,7 @@ struct bsdtar {
 /* Fake short equivalents for long options that otherwise lack them. */
 enum {
 	OPTION_AGGRESSIVE_NETWORKING=256,
+	OPTION_ARCHIVE_NAMES,
 	OPTION_CACHEDIR,
 	OPTION_CHECK_LINKS,
 	OPTION_CHECKPOINT_BYTES,
@@ -176,13 +196,16 @@ enum {
 	OPTION_DISK_PAUSE,
 	OPTION_DRYRUN,
 	OPTION_EXCLUDE,
+	OPTION_FORCE_RESOURCES,
 	OPTION_FSCK,
 	OPTION_FSCK_DELETE,	/* Operation mode, not a real option */
 	OPTION_FSCK_PRUNE,
 	OPTION_FSCK_WRITE,	/* Operation mode, not a real option */
 	OPTION_HELP,
 	OPTION_INCLUDE,
+	OPTION_INITIALIZE_CACHEDIR,
 	OPTION_INSANE_FILESYSTEMS,
+	OPTION_ISO_DATES,
 	OPTION_HUMANIZE_NUMBERS,
 	OPTION_KEYFILE,
 	OPTION_KEEP_GOING,
@@ -203,8 +226,10 @@ enum {
 	OPTION_NO_CONFIG_INCLUDE,
 	OPTION_NO_DEFAULT_CONFIG,
 	OPTION_NO_DISK_PAUSE,
+	OPTION_NO_FORCE_RESOURCES,
 	OPTION_NO_HUMANIZE_NUMBERS,
 	OPTION_NO_INSANE_FILESYSTEMS,
+	OPTION_NO_ISO_DATES,
 	OPTION_NO_MAXBW,
 	OPTION_NO_MAXBW_RATE_DOWN,
 	OPTION_NO_MAXBW_RATE_UP,
@@ -269,6 +294,7 @@ void	tarsnap_mode_r(struct bsdtar *bsdtar);
 void	tarsnap_mode_t(struct bsdtar *bsdtar);
 void	tarsnap_mode_x(struct bsdtar *bsdtar);
 void	tarsnap_mode_fsck(struct bsdtar *bsdtar, int prune, int whichkey);
+void	tarsnap_mode_initialize_cachedir(struct bsdtar *bsdtar);
 void	tarsnap_mode_list_archives(struct bsdtar *bsdtar);
 void	tarsnap_mode_nuke(struct bsdtar *bsdtar);
 void	tarsnap_mode_recover(struct bsdtar *bsdtar, int whichkey);
