@@ -6,9 +6,11 @@
 #include <assert.h>
 #include <errno.h>
 #include <limits.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "ccache_internal.h"
 #include "multitape_internal.h"
@@ -60,15 +62,15 @@ read_rec(void * cookie)
 		goto err0;
 
 	/* Decode record. */
-	ccr->ino = le64dec(ccre.ino);
-	ccr->size = le64dec(ccre.size);
-	ccr->mtime = le64dec(ccre.mtime);
-	ccr->nch = le64dec(ccre.nch);
+	ccr->ino = (ino_t)le64dec(ccre.ino);
+	ccr->size = (off_t)le64dec(ccre.size);
+	ccr->mtime = (time_t)le64dec(ccre.mtime);
+	ccr->nch = (size_t)le64dec(ccre.nch);
 	ccr->tlen = le32dec(ccre.tlen);
 	ccr->tzlen = le32dec(ccre.tzlen);
 	prefixlen = le32dec(ccre.prefixlen);
 	suffixlen = le32dec(ccre.suffixlen);
-	ccr->age = le32dec(ccre.age);
+	ccr->age = (int)le32dec(ccre.age);
 
 	/* Zero other fields. */
 	ccr->nchalloc = 0;
@@ -77,6 +79,11 @@ read_rec(void * cookie)
 	ccr->flags = 0;
 
 	/* Sanity check some fields. */
+	if (le64dec(ccre.nch) > (uint64_t)SIZE_MAX) {
+		warn0("Cache file is corrupt or too large for this "
+		    "platform: %s", R->s);
+		goto err1;
+	}
 	if ((prefixlen == 0 && suffixlen == 0) ||
 	    (ccr->nch > SIZE_MAX / sizeof(struct chunkheader)) ||
 	    (ccr->nch == 0 && ccr->tlen == 0) ||
@@ -138,8 +145,8 @@ callback_read_data(void * cookie, uint8_t * s, size_t slen, void * rec)
 	struct ccache_read_internal * R = cookie;
 	struct ccache_record * ccr = rec;
 
-	(void)s;	/* Unused. */
-	(void)slen;	/* Unused. */
+	(void)s;	/* UNUSED */
+	(void)slen;	/* UNUSED */
 
 	/* Read chunk headers, if present. */
 	if (ccr->nch) {
@@ -163,9 +170,9 @@ callback_free(void * cookie, uint8_t * s, size_t slen, void * rec)
 {
 	struct ccache_record * ccr = rec;
 
-	(void)cookie;	/* Unused. */
-	(void)s;	/* Unused. */
-	(void)slen;	/* Unused. */
+	(void)cookie;	/* UNUSED */
+	(void)s;	/* UNUSED */
+	(void)slen;	/* UNUSED */
 
 	/* Free chunkheader records, if they weren't mmapped. */
 	if (ccr->nchalloc)
@@ -280,12 +287,13 @@ ccache_read(const char * path)
 		warnp("fstat(%s)", R.s);
 		goto err5;
 	}
-	if (sb.st_size != (off_t)(fpos + R.datalen)) {
+	if (sb.st_size != fpos + (off_t)R.datalen) {
 		warn0("Cache has incorrect size (%jd, expected %jd)\n",
-		    (intmax_t)(sb.st_size), (intmax_t)(fpos + R.datalen));
+		    (intmax_t)(sb.st_size),
+		    (intmax_t)(fpos + (off_t)R.datalen));
 		goto err5;
 	}
-	C->datalen = R.datalen + (fpos % pagesize);
+	C->datalen = R.datalen + (size_t)(fpos % pagesize);
 	if ((C->data = mmap(NULL, C->datalen, PROT_READ,
 #ifdef MAP_NOCORE
 	    MAP_PRIVATE | MAP_NOCORE,
