@@ -4,9 +4,11 @@
 
 #include <assert.h>
 #include <errno.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "ccache_internal.h"
 #include "multitape_internal.h"
@@ -67,6 +69,12 @@ callback_write_rec(void * cookie, uint8_t * s, size_t slen, void * rec)
 	struct ccache_record * ccr = rec;
 	size_t plen;
 
+	/* Sanity checks. */
+	assert(slen <= UINT32_MAX);
+	assert((ccr->size >= 0) && ((uintmax_t)ccr->size <= UINT64_MAX));
+	assert((ccr->mtime >= 0) && ((uintmax_t)ccr->mtime <= UINT64_MAX));
+	assert((uintmax_t)ccr->ino <= UINT64_MAX);
+
 	/* Don't write an entry if there are no chunks and no trailer. */
 	if ((ccr->nch == 0) && (ccr->tlen == 0))
 		goto done;
@@ -82,15 +90,15 @@ callback_write_rec(void * cookie, uint8_t * s, size_t slen, void * rec)
 	}
 
 	/* Convert integers to portable format. */
-	le64enc(ccre.ino, ccr->ino);
-	le64enc(ccre.size, ccr->size);
-	le64enc(ccre.mtime, ccr->mtime);
-	le64enc(ccre.nch, ccr->nch);
-	le32enc(ccre.tlen, ccr->tlen);
-	le32enc(ccre.tzlen, ccr->tzlen);
-	le32enc(ccre.prefixlen, plen);
-	le32enc(ccre.suffixlen, slen - plen);
-	le32enc(ccre.age, ccr->age + 1);
+	le64enc(ccre.ino, (uint64_t)ccr->ino);
+	le64enc(ccre.size, (uint64_t)ccr->size);
+	le64enc(ccre.mtime, (uint64_t)ccr->mtime);
+	le64enc(ccre.nch, (uint64_t)ccr->nch);
+	le32enc(ccre.tlen, (uint32_t)ccr->tlen);
+	le32enc(ccre.tzlen, (uint32_t)ccr->tzlen);
+	le32enc(ccre.prefixlen, (uint32_t)plen);
+	le32enc(ccre.suffixlen, (uint32_t)(slen - plen));
+	le32enc(ccre.age, (uint32_t)(ccr->age + 1));
 
 	/* Write cache entry header to disk. */
 	if (fwrite(&ccre, sizeof(ccre), 1, W->f) != 1)
@@ -206,8 +214,15 @@ ccache_write(CCACHE * cache, const char * path)
 		goto err2;
 	};
 
+	/* Check that we don't have too many cache records. */
+	if (W.N > UINT32_MAX) {
+		warn0("Programmer error: "
+		    "The cache cannot contain more than 2^32-1 entries");
+		goto err2;
+	}
+
 	/* Write the number of records to the file. */
-	le32enc(N, W.N);
+	le32enc(N, (uint32_t)W.N);
 	if (fwrite(N, 4, 1, W.f) != 1) {
 		warnp("fwrite(%s)", W.s);
 		goto err2;
