@@ -1,5 +1,7 @@
 #include "bsdtar_platform.h"
 
+#include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
