@@ -1,5 +1,9 @@
 #include "bsdtar_platform.h"
 
+#include <errno.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -335,8 +339,8 @@ chunks_stats_addchunk(CHUNKS_S * C, const uint8_t * hash,
 
 	/* Record the lengths if necessary. */
 	if (ch->d.nrefs == 0 && ch->ncopies_ctape == 0) {
-		ch->d.len = len;
-		ch->d.zlen_flags = zlen | (ch->d.zlen_flags & CHDATA_FLAGS);
+		ch->d.len = (uint32_t)len;
+		ch->d.zlen_flags = (uint32_t)(zlen | (ch->d.zlen_flags & CHDATA_FLAGS));
 	}
 
 	/* Update "current tape" statistics. */
@@ -414,3 +418,51 @@ chunks_stats_free(CHUNKS_S * C)
 	free(C->cachepath);
 	free(C);
 }
+
+/**
+ * chunks_initialize(const char * cachepath):
+ * Initialize the chunk directory (file) in ${cachepath}.  Return 0 on
+ * success, -1 on error, and 1 if the file already exists.
+ */
+int
+chunks_initialize(const char * cachepath)
+{
+	RWHASHTAB * HT;
+	struct chunkstats stats_extra;
+
+	/* Bail if ${chunkpath}/directory already exists. */
+ 	switch (chunks_directory_exists(cachepath)) {
+	case 0:
+		break;
+	case 1:
+		goto fileexists;
+	case -1:
+		goto err0;
+	}
+
+	/* Allocate empty hash table, and zero stats. */
+	if ((HT = rwhashtab_init(0, 1)) == NULL)
+		goto err0;
+	chunks_stats_zero(&stats_extra);
+
+	/* Write empty directory file. */
+	if (chunks_directory_write(cachepath, HT, &stats_extra, ""))
+		goto err1;
+
+	/* Free memory. */
+	rwhashtab_free(HT);
+
+	/* Success! */
+	return (0);
+
+err1:
+	rwhashtab_free(HT);
+err0:
+	/* Failure! */
+	return (-1);
+
+fileexists:
+
+	/* Failure! */
+	return (1);
+}
