@@ -1,7 +1,9 @@
 #include "bsdtar_platform.h"
 
+#include <assert.h>
 #include <errno.h>
 #include <limits.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -25,6 +27,7 @@ struct chunks_write_internal {
 	struct chunkstats stats_total;	/* All archives, w/ multiplicity. */
 	struct chunkstats stats_unique;	/* All archives, w/o multiplicity. */
 	struct chunkstats stats_extra;	/* Extra (non-chunked) data. */
+	struct chunkstats stats_extra_copy;	/* Copy for checkpoint stats. */
 	struct chunkstats stats_tape;	/* This archive, w/ multiplicity. */
 	struct chunkstats stats_new;	/* New chunks. */
 	struct chunkstats stats_tapee;	/* Extra data in this archive. */
@@ -110,6 +113,10 @@ chunks_write_chunk(CHUNKS_W * C, const uint8_t * hash,
 	char hashbuf[65];
 	int rc;
 
+	/* Sanity checks. */
+	assert(buflen <= UINT32_MAX);
+	assert(C->zbuflen < CHDATA_MALLOC);
+
 	/* If the chunk is in ${C}->HT, return the compressed length. */
 	if ((ch = rwhashtab_read(C->HT, hash)) != NULL) {
 		chunks_stats_add(&C->stats_total, ch->len,
@@ -144,6 +151,12 @@ chunks_write_chunk(CHUNKS_W * C, const uint8_t * hash,
 		goto err0;
 	}
 
+	/* Sanity check the compressed size. */
+	if (zlen > SSIZE_MAX) {
+		warnp("Error compressing chunk");
+		goto err0;
+	}
+
 	/* Ask the storage layer to write the file for us. */
 	if (storage_write_file(C->S, C->zbuf, zlen, 'c', hash)) {
 		hexify(hash, hashbuf, 32);
@@ -157,8 +170,8 @@ chunks_write_chunk(CHUNKS_W * C, const uint8_t * hash,
 
 	/* ... fill in the chunk parameters... */
 	memcpy(ch->hash, hash, 32);
-	ch->len = buflen;
-	ch->zlen_flags = zlen | CHDATA_MALLOC | CHDATA_CTAPE;
+	ch->len = (uint32_t)buflen;
+	ch->zlen_flags = (uint32_t)(zlen | CHDATA_MALLOC | CHDATA_CTAPE);
 	ch->nrefs = 1;
 	ch->ncopies = 1;
 
@@ -173,7 +186,7 @@ chunks_write_chunk(CHUNKS_W * C, const uint8_t * hash,
 	chunks_stats_add(&C->stats_new, ch->len, zlen, 1);
 
 	/* Success! */
-	return (zlen);
+	return ((ssize_t)zlen);
 
 err1:
 	free(ch);
@@ -243,6 +256,21 @@ chunks_write_extrastats(CHUNKS_W * C, size_t len)
 	chunks_stats_add(&C->stats_tapee, len, len, 1);
 }
 
+/**
+ * chunks_write_extrastats_copy(C, direction):
+ * Make a copy of the extra stats; if ${direction} is 0, copy from the real
+ * stats to a copy; if 1, set the real stats to the copy.
+ */
+void
+chunks_write_extrastats_copy(CHUNKS_W * C, size_t direction)
+{
+
+	if (direction == 0)
+		C->stats_extra_copy = C->stats_extra;
+	else
+		C->stats_extra = C->stats_extra_copy;
+}
+
 /**
  * chunks_write_printstats(stream, C, csv):
  * Print statistics for the write transaction associated with the cookie
