@@ -26,7 +26,7 @@ tarsnap_mode_d(struct bsdtar *bsdtar)
 
 	/* Delete archives. */
 	for (i = 0; i < bsdtar->ntapes; i++) {
-		if (bsdtar->verbose && (bsdtar->ntapes > 1))
+		if (bsdtar->verbose)
 			fprintf(stderr, "Deleting archive \"%s\"\n",
 			    bsdtar->tapenames[i]);
 		switch (deletetape(d, bsdtar->machinenum, bsdtar->cachedir,
@@ -231,6 +231,32 @@ err0:
 	return;
 }
 
+/*
+ * Initialize cache directory.
+ */
+void
+tarsnap_mode_initialize_cachedir(struct bsdtar *bsdtar)
+{
+
+	switch (statstape_initialize(bsdtar->machinenum, bsdtar->cachedir)) {
+	case 0:
+		break;
+	case 1:
+		bsdtar_warnc(bsdtar, 0, "Cache directory already initialized");
+		/* FALLTHROUGH */
+	default:
+		goto err0;
+	}
+
+	/* Success! */
+	return;
+
+err0:
+	/* Failure! */
+	bsdtar->return_value = 1;
+	return;
+}
+
 /*
  * Nuke all the files belonging to an archive set.
  */
