@@ -1,12 +1,13 @@
 #include "bsdtar_platform.h"
 
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "chunks.h"
 #include "crypto.h"
-#include "dirutil.h"
 #include "hexify.h"
 #include "imalloc.h"
 #include "multitape_internal.h"
@@ -122,7 +123,7 @@ deletearchive(STORAGE_D * SD, struct tapemetadata * tmd)
 
 	/* Delete index fragments. */
 	for (fragnum = 0; fragnum * MAXIFRAG < tmd->indexlen; fragnum++) {
-		multitape_metaindex_fragname(hbuf, fragnum, fraghash);
+		multitape_metaindex_fragname(hbuf, (uint32_t)fragnum, fraghash);
 		if (storage_delete_file(SD, 'i', fraghash))
 			goto err0;
 	}
@@ -269,7 +270,8 @@ phase2(uint64_t machinenum, STORAGE_D * SD,
 		for (fragnum = 0;
 		    fragnum * MAXIFRAG < mdatlist[file]->indexlen;
 		    fragnum++) {
-			multitape_metaindex_fragname(hbuf, fragnum, fraghash);
+			multitape_metaindex_fragname(hbuf, (uint32_t)fragnum,
+			    fraghash);
 			if (findinlist(fraghash, flist, nfiles) == nfiles) {
 				fprintf(stdout,
 				    "  Deleting archive"
@@ -308,7 +310,8 @@ nextfile:	;
 		for (fragnum = 0;
 		    fragnum * MAXIFRAG < mdatlist[file]->indexlen;
 		    fragnum++) {
-			multitape_metaindex_fragname(hbuf, fragnum, fraghash);
+			multitape_metaindex_fragname(hbuf, (uint32_t)fragnum,
+			    fraghash);
 			neededvec[findinlist(fraghash, flist, nfiles)] = 1;
 		}
 	}
@@ -461,18 +464,14 @@ fscktape(uint64_t machinenum, const char * cachedir, int prune, int whichkey)
 	struct tapemetadata ** mdatlist;
 	size_t nmdat;
 	size_t file;
-	int key = (whichkey == 0) ? 0 : 1;
-
-	/* Make sure the cache directory exists. */
-	if (dirutil_needdir(cachedir))
-		goto err0;
+	uint8_t key = (whichkey == 0) ? 0 : 1;
 
 	/* Lock the cache directory. */
 	if ((lockfd = multitape_lock(cachedir)) == -1)
 		goto err0;
 
 	/* Make sure the lower layers are in a clean state. */
-	if (multitape_cleanstate(cachedir, machinenum, whichkey))
+	if (multitape_cleanstate(cachedir, machinenum, key))
 		goto err1;
 
 	/*
@@ -573,3 +572,28 @@ err0:
 	/* Failure! */
 	return (-1);
 }
+
+/**
+ * statstape_initialize(machinenum, cachedir):
+ * Initialize an empty chunk directory in ${cachedir} so that --print-stats
+ * works.  This requires the "directory" file, but no other files.  Return 0
+ * on success, -1 on error, and 1 if the cachedir is already initialized.
+ */
+int
+statstape_initialize(uint64_t machinenum, const char * cachedir)
+{
+	int rc;
+
+	(void) machinenum; /* UNUSED */
+
+	/* Initialize the "directory" file. */
+	if ((rc = chunks_initialize(cachedir)) != 0)
+		goto err0;
+
+	/* Success! */
+	return (0);
+
+err0:
+	/* Failure! */
+	return (rc);
+}
