@@ -2,6 +2,7 @@
 
 #include <sys/types.h>
 
+#include <assert.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -68,6 +69,9 @@ multitape_metadata_enc(const struct tapemetadata * mdat, uint8_t ** bufp,
 	uint8_t * p;
 	int i;
 
+	/* Sanity check. */
+	assert((mdat->argc >= 0) && ((uintmax_t)mdat->argc <= UINT32_MAX));
+
 	/* Add up the lengths of various pieces of metadata. */
 	buflen = strlen(mdat->name) + 1;	/* name */
 	buflen += 8;				/* ctime */
@@ -87,9 +91,9 @@ multitape_metadata_enc(const struct tapemetadata * mdat, uint8_t ** bufp,
 	p += strlen(mdat->name) + 1;
 
 	/* Encode ctime and argc. */
-	le64enc(p, mdat->ctime);
+	le64enc(p, (uint64_t)mdat->ctime);
 	p += 8;
-	le32enc(p, mdat->argc);
+	le32enc(p, (uint32_t)mdat->argc);
 	p += 4;
 
 	/* Copy argv. */
@@ -107,7 +111,8 @@ multitape_metadata_enc(const struct tapemetadata * mdat, uint8_t ** bufp,
 	p += 8;
 
 	/* Generate signature. */
-	if (crypto_rsa_sign(CRYPTO_KEY_SIGN_PRIV, buf, p - buf, p, 256))
+	if (crypto_rsa_sign(CRYPTO_KEY_SIGN_PRIV, buf, (size_t)(p - buf), p,
+	    256))
 		goto err1;
 
 	/* Return buffer and length. */
@@ -125,13 +130,13 @@ err0:
 }
 
 /**
- * multitape_metadata_put(S, C, mdat, extrastats):
+ * multitape_metadata_put(S, C, mdat):
  * Store archive metadata.  Call chunks_write_extrastats on ${C} and the
- * metadata file length if ${extrastats} != 0.
+ * metadata file length.
  */
 int
 multitape_metadata_put(STORAGE_W * S, CHUNKS_W * C,
-    struct tapemetadata * mdat, int extrastats)
+    struct tapemetadata * mdat)
 {
 	uint8_t hbuf[32];	/* HMAC of tape name. */
 	uint8_t * buf;		/* Encoded metadata. */
@@ -149,8 +154,7 @@ multitape_metadata_put(STORAGE_W * S, CHUNKS_W * C,
 	/* Store the archive metadata. */
 	if (storage_write_file(S, buf, buflen, 'm', hbuf))
 		goto err1;
-	if (extrastats)
-		chunks_write_extrastats(C, buflen);
+	chunks_write_extrastats(C, buflen);
 
 	/* Free metadata buffer. */
 	free(buf);
@@ -197,21 +201,22 @@ multitape_metadata_dec(struct tapemetadata * mdat, uint8_t * buf,
 	/* Parse ctime and argc. */
 	if (buflen < 8)
 		goto bad1;
-	mdat->ctime = le64dec(p);
+	mdat->ctime = (time_t)le64dec(p);
 	buflen -= 8;
 	p += 8;
 	if (buflen < 4)
 		goto bad1;
-	mdat->argc = le32dec(p);
+	mdat->argc = (int)le32dec(p);
 	buflen -= 4;
 	p += 4;
 
 	/* Sanity-check argc. */
-	if ((mdat->argc < 0) || ((size_t)(mdat->argc) > buflen))
+	if ((mdat->argc < 0) || ((uintmax_t)mdat->argc > SIZE_MAX) ||
+	    ((size_t)(mdat->argc) > buflen))
 		goto bad1;
 
 	/* Allocate space for argv. */
-	if ((mdat->argv = malloc(mdat->argc * sizeof(char *))) == NULL)
+	if ((mdat->argv = malloc((size_t)mdat->argc * sizeof(char *))) == NULL)
 		goto err1;
 
 	/* Parse argv. */
@@ -250,7 +255,7 @@ multitape_metadata_dec(struct tapemetadata * mdat, uint8_t * buf,
 	if (buflen < 256)
 		goto bad2;
 	switch (crypto_rsa_verify(CRYPTO_KEY_SIGN_PUB,
-	    buf, p - buf, p, 256)) {
+	    buf, (size_t)(p - buf), p, 256)) {
 	case -1:
 		/* Error in crypto_rsa_verify. */
 		goto err2;
