@@ -2,6 +2,7 @@
 
 #include <sys/types.h>
 
+#include <errno.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,14 +45,14 @@ multitape_metaindex_fragname(const uint8_t namehash[32], uint32_t fragnum,
 }
 
 /**
- * multitape_metaindex_put(S, C, mind, mdat, extrastats):
+ * multitape_metaindex_put(S, C, mind, mdat):
  * Store the provided archive metaindex, and update the archive metadata
  * with the metaindex parameters.  Call chunks_write_extrastats on ${C} and
- * the length(s) of file(s) containing the metaindex if ${extrastats} != 0.
+ * the length(s) of file(s) containing the metaindex.
  */
 int
 multitape_metaindex_put(STORAGE_W * S, CHUNKS_W * C,
-    struct tapemetaindex * mind, struct tapemetadata * mdat, int extrastats)
+    struct tapemetaindex * mind, struct tapemetadata * mdat)
 {
 	uint8_t hbuf[32];	/* HMAC of tape name. */
 	uint8_t * buf;		/* Encoded metaindex. */
@@ -79,7 +80,7 @@ multitape_metaindex_put(STORAGE_W * S, CHUNKS_W * C,
 		errno = ENOMEM;
 		goto err0;
 	} else {
-		buflen = totalsize;
+		buflen = (size_t)totalsize;
 	}
 
 	/* Allocate memory. */
@@ -87,19 +88,19 @@ multitape_metaindex_put(STORAGE_W * S, CHUNKS_W * C,
 		goto err0;
 
 	/* Copy values into buffer. */
-	le32enc(p, mind->hindexlen);
+	le32enc(p, (uint32_t)mind->hindexlen);
 	p += 4;
 	if (mind->hindexlen > 0)
 		memcpy(p, mind->hindex, mind->hindexlen);
 	p += mind->hindexlen;
 
-	le32enc(p, mind->cindexlen);
+	le32enc(p, (uint32_t)mind->cindexlen);
 	p += 4;
 	if (mind->cindexlen > 0)
 		memcpy(p, mind->cindex, mind->cindexlen);
 	p += mind->cindexlen;
 
-	le32enc(p, mind->tindexlen);
+	le32enc(p, (uint32_t)mind->tindexlen);
 	p += 4;
 	if (mind->tindexlen > 0)
 		memcpy(p, mind->tindex, mind->tindexlen);
@@ -115,12 +116,11 @@ multitape_metaindex_put(STORAGE_W * S, CHUNKS_W * C,
 		fraglen = buflen - fragnum * MAXIFRAG;
 		if (fraglen > MAXIFRAG)
 			fraglen = MAXIFRAG;
-		multitape_metaindex_fragname(hbuf, fragnum, fraghash);
+		multitape_metaindex_fragname(hbuf, (uint32_t)fragnum, fraghash);
 		if (storage_write_file(S, buf + fragnum * MAXIFRAG,
 		    fraglen, 'i', fraghash))
 			goto err1;
-		if (extrastats)
-			chunks_write_extrastats(C, fraglen);
+		chunks_write_extrastats(C, fraglen);
 	}
 
 	/* Compute the hash of the metaindex. */
@@ -179,15 +179,15 @@ multitape_metaindex_get(STORAGE_R * S, CHUNKS_S * C,
 		errno = ENOMEM;
 		goto err0;
 	}
-	if ((mbuf = malloc(mdat->indexlen)) == NULL)
+	if ((mbuf = malloc((size_t)mdat->indexlen)) == NULL)
 		goto err0;
 
 	/* Read the archive metaindex. */
 	for (fragnum = 0; fragnum * MAXIFRAG < mdat->indexlen; fragnum++) {
-		fraglen = mdat->indexlen - fragnum * MAXIFRAG;
+		fraglen = (size_t)mdat->indexlen - fragnum * MAXIFRAG;
 		if (fraglen > MAXIFRAG)
 			fraglen = MAXIFRAG;
-		multitape_metaindex_fragname(hbuf, fragnum, fraghash);
+		multitape_metaindex_fragname(hbuf, (uint32_t)fragnum, fraghash);
 		switch (storage_read_file(S, mbuf + fragnum * MAXIFRAG,
 		    fraglen, 'i', fraghash)) {
 		case -1:
@@ -207,7 +207,7 @@ multitape_metaindex_get(STORAGE_R * S, CHUNKS_S * C,
 
 	/* Make sure the index matches the hash provided. */
 	if (crypto_hash_data(CRYPTO_KEY_HMAC_SHA256,
-	    mbuf, mdat->indexlen, indexhbuf)) {
+	    mbuf, (size_t)mdat->indexlen, indexhbuf)) {
 		warnp("Programmer error: "
 		    "SHA256 should never fail");
 		goto err1;
@@ -217,7 +217,7 @@ multitape_metaindex_get(STORAGE_R * S, CHUNKS_S * C,
 
 	/* We haven't parsed any of the index yet. */
 	buf = mbuf;
-	buflen = mdat->indexlen;
+	buflen = (size_t)mdat->indexlen;
 
 	/* Extract header stream index. */
 	if (buflen < 4)
@@ -332,7 +332,7 @@ multitape_metaindex_free(struct tapemetaindex * mind)
 /**
  * multitape_metaindex_delete(S, C, mdat):
  * Delete the metaindex file associated with the provided metadata.  Call
- * chnks_delete_extrastats on ${C} and the length(s) of file(s) containing
+ * chunks_delete_extrastats on ${C} and the length(s) of file(s) containing
  * the metaindex.
  */
 int
@@ -349,10 +349,10 @@ multitape_metaindex_delete(STORAGE_D * S, CHUNKS_D * C,
 		goto err0;
 
 	for (fragnum = 0; fragnum * MAXIFRAG < mdat->indexlen; fragnum++) {
-		fraglen = mdat->indexlen - fragnum * MAXIFRAG;
+		fraglen = (size_t)mdat->indexlen - fragnum * MAXIFRAG;
 		if (fraglen > MAXIFRAG)
 			fraglen = MAXIFRAG;
-		multitape_metaindex_fragname(hbuf, fragnum, fraghash);
+		multitape_metaindex_fragname(hbuf, (uint32_t)fragnum, fraghash);
 		if (storage_delete_file(S, 'i', fraghash))
 			goto err0;
 		chunks_delete_extrastats(C, fraglen);
