@@ -1,15 +1,18 @@
 #include "bsdtar_platform.h"
 
 #include <assert.h>
+#include <limits.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "chunkify.h"
 #include "chunks.h"
 #include "crypto.h"
-#include "dirutil.h"
+#include "ctassert.h"
 #include "elasticarray.h"
 #include "storage.h"
 #include "sysendian.h"
@@ -19,6 +22,9 @@
 
 #include "multitape.h"
 
+/* This API relies upon chunks being no more than SSIZE_MAX bytes in length. */
+CTASSERT(MAXCHUNK <= SSIZE_MAX);
+
 /* Mean chunk size desired. */
 #define	MEANCHUNK	65536
 
@@ -92,14 +98,14 @@ static chunkify_callback callback_t;
 static chunkify_callback callback_c;
 static chunkify_callback callback_file;
 static int endentry(TAPE_W *);
-static int flushtape(TAPE_W *, int, int);
+static int flushtape(TAPE_W *, int);
 
 /* Initialize stream. */
 static int
 stream_init(struct stream * S, chunkify_callback callback, void * cookie)
 {
 
-	/* Create chunkfier. */
+	/* Create chunkifier. */
 	if ((S->c =
 	    chunkify_init(MEANCHUNK, MAXCHUNK, callback, cookie)) == NULL)
 		goto err0;
@@ -351,14 +357,18 @@ endentry(TAPE_W * d)
 	if (bytebuf_export(d->hbuf, &hbuf, &hlen))
 		goto err0;
 
+	/* Sanity checks. */
+	assert(hlen < UINT32_MAX);
+	assert((d->clen >= 0) && ((uintmax_t)d->clen <= UINT64_MAX));
+
 	/* Create a new elastic archive header buffer. */
 	if ((d->hbuf = bytebuf_init(0)) == NULL)
 		goto err1;
 
 	/* Construct entry header. */
-	le32enc(eh.hlen, hlen);
-	le64enc(eh.clen, d->clen);
-	le32enc(eh.tlen, d->tlen);
+	le32enc(eh.hlen, (uint32_t)hlen);
+	le64enc(eh.clen, (uint64_t)d->clen);
+	le32enc(eh.tlen, (uint32_t)d->tlen);
 
 	/* Write entry header to header stream. */
 	if (chunkify_write(d->h.c, (uint8_t *)(&eh),
@@ -443,10 +453,6 @@ writetape_open(uint64_t machinenum, const char * cachedir,
 	/* Record whether to print statistics to a CSV file. */
 	d->csv_filename = csv_filename;
 
-	/* If we're using a cache, make sure ${cachedir} exists. */
-	if ((cachedir != NULL) && (dirutil_needdir(cachedir)))
-		goto err3;
-
 	/* If we're using a cache, lock the cache directory. */
 	if ((cachedir != NULL) && ((d->lockfd = multitape_lock(cachedir)) == -1))
 		goto err3;
@@ -497,7 +503,7 @@ writetape_open(uint64_t machinenum, const char * cachedir,
 	if ((d->hbuf = bytebuf_init(0)) == NULL)
 		goto err9;
 
-	/* Iinitialize file chunkifier. */
+	/* Initialize file chunkifier. */
 	if ((d->c_file = chunkify_init(MEANCHUNK, MAXCHUNK, &callback_file,
 	    (void *)d)) == NULL)
 		goto err10;
@@ -563,6 +569,9 @@ ssize_t
 writetape_write(TAPE_W * d, const void * buffer, size_t nbytes)
 {
 
+	/* Sanity check */
+	assert(nbytes <= SSIZE_MAX);
+
 	/* Don't write anything if we're truncating the archive. */
 	if (d->eof)
 		goto eof;
@@ -588,7 +597,7 @@ writetape_write(TAPE_W * d, const void * buffer, size_t nbytes)
 	}
 
 	/* Success! */
-	return (nbytes);
+	return ((ssize_t)nbytes);
 
 eof:
 	/* Archive is being truncated; refuse to write anything. */
@@ -608,7 +617,7 @@ writetape_ischunkpresent(TAPE_W * d, struct chunkheader * ch)
 {
 
 	if (chunks_write_ispresent(d->C, ch->hash) == 0)
-		return (le32dec(ch->len));
+		return ((ssize_t)le32dec(ch->len));
 	else
 		return (0);
 }
@@ -656,7 +665,7 @@ writetape_writechunk(TAPE_W * d, struct chunkheader * ch)
 	d->clen += le32dec(ch->len);
 
 	/* Success! */
-	return (le32dec(ch->len));
+	return ((ssize_t)le32dec(ch->len));
 
 notpresent:
 	/* The chunk layer doesn't have a chunk with this hash. */
@@ -721,13 +730,12 @@ writetape_truncate(TAPE_W * d)
 }
 
 /**
- * flushtape(d, isapart, extrastats):
+ * flushtape(d, isapart):
  * Flush the chunkifiers, store metaindex and metadata, and issue a flush at
- * the storage layer.  If ${extrastats} is non-zero, add the metaindex and
- * metadata sizes to storage statistics.
+ * the storage layer.
  */
 static int
-flushtape(TAPE_W * d, int isapart, int extrastats)
+flushtape(TAPE_W * d, int isapart)
 {
 	struct tapemetadata tmd;
 	struct tapemetaindex tmi;
@@ -783,11 +791,11 @@ flushtape(TAPE_W * d, int isapart, int extrastats)
 	 * archive metadata is stored, since it fills in fields in the archive
 	 * metadata concerning the index length and hash.
 	 */
-	if (multitape_metaindex_put(d->S, d->C, &tmi, &tmd, extrastats))
+	if (multitape_metaindex_put(d->S, d->C, &tmi, &tmd))
 		goto err4;
 
 	/* Store archive metadata. */
-	if (multitape_metadata_put(d->S, d->C, &tmd, extrastats))
+	if (multitape_metadata_put(d->S, d->C, &tmd))
 		goto err4;
 
 	/* Free duplicated chunk indexes. */
@@ -847,17 +855,35 @@ writetape_checkpoint(TAPE_W * d)
 	if ((d->mode == 3) && endentry(d))
 		goto err0;
 
+	/*
+	 * Back up archive set statistics before adding the metadata and
+	 * metaindex; these will be restored after the chunk layer writes the
+	 * directory file.
+	 */
+	chunks_write_extrastats_copy(d->C, 0);
+
 	/*
 	 * Flush data through and write the metaindex and metadata;
 	 * checkpoints are partial archives, so mark it as such.
+	 * This also adds the metadata and metaindex to the "extra"
+	 * statistics; we need these when we write the directory file, but we
+	 * will restore the original statistics later since the metadata and
+	 * metaindex from this checkpoint will be discarded if/when another
+	 * checkpoint is created or the archive is completed.
 	 */
-	if (flushtape(d, 1, 0))
+	if (flushtape(d, 1))
 		goto err0;
 
 	/* Ask the chunks layer to prepare for a checkpoint. */
 	if (chunks_write_checkpoint(d->C))
 		goto err0;
 
+	/*
+	 * Restore original statistics (i.e. without the metadata and
+	 * metaindex).
+	 */
+	chunks_write_extrastats_copy(d->C, 1);
+
 	/* If this isn't a dry run, create a checkpoint. */
 	if ((d->S != NULL) &&
 	    multitape_checkpoint(d->cachedir, d->machinenum, d->seqnum))
@@ -911,7 +937,7 @@ writetape_close(TAPE_W * d)
 	}
 
 	/* Flush data through and write the metaindex and metadata. */
-	if (flushtape(d, d->eof, 1))
+	if (flushtape(d, d->eof))
 		goto err2;
 
 	/* Print statistics, if we've been asked to do so. */
